import 'package:flutter/cupertino.dart';

import 'AbstractServerData.dart';
import 'RespondError.dart';

/// The response of an operation in the server.
class ResponseCli extends AbstractServerData{
  static final type = '_class_type_response';
  final _class_type_response = '_';
  String _clientRequestId;

  /// Request ID generated by the client.
  String get clientRequestId => _clientRequestId;

  /// Result of operation in the server.
  ///
  /// Do NOT use this field tho check if the operation
  /// failed (because it can be null even in case of success),
  /// instead use  [isSuccess]
  dynamic output;

  /// The response error in case where [isSuccess == false]
  RespondError error;

  ResponseCli({@required String clientRequestId, this.output, this.error}):super(){
    this._clientRequestId = clientRequestId;
  }

  ResponseCli.fromMap(map){
    setValuesFromMap(map);
  }

  /// Returns [true] if the response is a success.
  bool get isSuccess => this.error == null;

  void setValuesFromMap(map) {
    super.fromMap(map);
    this._clientRequestId = map['clientRequestId'];
    this.output = map['output'];
    this.error = map['error'] != null ? RespondError.fromMap(map['error']) : null;
  }
}
